{$R-,S+,I+,D+,T-,F-,V+,B-,N+,L- }
unit graphic;

interface

uses
  normal;

const
  WINDOW_FULL = -1;

type
{$IFOPT N+}
  real = extended;
{$ENDIF}
  rectangle = record
    left, right, bottom, top: real;
  end;
  windows = integer;

procedure SetRectangle(var rect: rectangle; l, b, r, t: real);
function  graphic_openwindow(new_viewport, new_userworld: rectangle): windows;
procedure graphic_closewindow(w: integer);
procedure graphic_pset(w: windows; x, y: real; c: byte);
procedure graphic_line(w: windows; x1, y1, x2, y2: real; c: byte);
procedure graphic_cls;
procedure graphic_clearwindow(w: windows);

implementation

const
  NUMBER_OF_GRAPHIC_WINDOWS = 4;

type
  graphic_window = record
    used: boolean;
    viewport: rectangle;
    userworld: rectangle;
    ratio_x,
    ratio_y: real;
  end;

var
  windows_array: array [1..NUMBER_OF_GRAPHIC_WINDOWS] of graphic_window;
  DoClipping: boolean;
  i: integer;

procedure SetRectangle(var rect: rectangle; l, b, r, t: real);
  begin
    with rect do begin
      left := l; bottom := b; right := r; top := t;
    end;
  end;

function graphic_openwindow(new_viewport, new_userworld: rectangle): windows;
  var
    i: integer;
  function width(r: rectangle): real;
    begin
      with r do width := right - left;
    end;
  function height(r: rectangle): real;
    begin
      with r do height := top - bottom;
    end;
  begin
    for i := 1 to NUMBER_OF_GRAPHIC_WINDOWS do
      with windows_array[i] do
        if not used then begin
          used := True;
          viewport := new_viewport;
          userworld := new_userworld;
          ratio_x := width(viewport) / width(userworld);
          ratio_y := height(viewport) / width(userworld);
          graphic_openwindow := i;
          exit;
        end;
    graphic_openwindow := WINDOW_FULL;
  end;

procedure graphic_closewindow(w: windows);
  begin
    if (1 <= w) and (w <= NUMBER_OF_GRAPHIC_WINDOWS) then
      windows_array[w].used := False;
  end;

function normal_x(w: windows; x: real): real;
  begin
    with windows_array[w] do begin
      normal_x := viewport.left + ratio_x * (x - userworld.left);
    end;
  end;

function normal_y(w: windows; y: real): real;
  begin
    with windows_array[w] do begin
      normal_y := viewport.bottom + ratio_y * (y - userworld.bottom);
    end;
  end;

procedure graphic_pset(w: windows; x, y: real; c: byte);
  begin
    if DoClipping then
      with windows_array[w].userworld do
        if (x < left) or (x > right) or (y < bottom) or (y > top) then exit;
    normal_pset(normal_x(w, x), normal_y(w, y), c);
  end;

procedure graphic_clippingline(w: windows; x1, y1, x2, y2: real; c: byte);
  var
    flag1, flag2: integer;
  function comp_flag(x, y: real): integer;
    var
      flag, flag1, flag2: integer;
    begin
      with windows_array[w].userworld do begin
        flag := 0;
        if y > top then
          flag := 1
        else if y < bottom then
          flag := 2;
        if x > right then
          flag := flag + 4
        else if x < left then
          flag := flag + 8;
      end;
      comp_flag := flag;
    end;
  procedure sublinew(var x1, y1, x2, y2: real; flag: integer);
    begin
      with windows_array[w].userworld do begin
        if (flag and 1) <> 0 then begin
          x2 := x1 + ((x2 - x1) / (y2 - y1)) * (top - y1);
          y2 := top;
          flag := comp_flag(x2,y2);
        end
        else if (flag and 2) <> 0 then begin
          x2 := x1 + ((x2 - x1) / (y2 - y1)) * (bottom - y1);
          y2 := bottom;
          flag := comp_flag(x2,y2);
        end;

        if (flag and 4) <> 0 then begin
          y2 := y1 + ((y2 - y1) / (x2 - x1)) * (right - x1);
          x2 := right;
        end
        else if (flag and 8) <> 0 then begin
          y2 := y1 + ((y2 - y1) / (x2 - x1)) * (left - x1);
          x2 := left;
        end;
      end{ with }
    end;
  begin { main part of ClippingLine }
    flag1 := comp_flag(x1, y1);
    flag2 := comp_flag(x2, y2);
    if (flag1 and flag2) <> 0 then exit;
    if flag1 = 0 then begin
      if flag2 <> 0  then sublinew(x1, y1, x2, y2, flag2);
      normal_line(normal_x(w, x1), normal_y(w, y1),
                  normal_x(w, x2), normal_y(w, y2), c);
    end
    else if flag2 = 0 then begin
      sublinew(x2, y2, x1, y1, flag1);
      normal_line(normal_x(w, x1), normal_y(w, y1),
                  normal_x(w, x2), normal_y(w, y2), c);
    end
    else begin
      sublinew(x2, y2, x1, y1, flag1);
      flag1 := comp_flag(x1, y1);
      if flag1 = 0 then begin
        sublinew(x1, y1, x2, y2, flag2);
        normal_line(normal_x(w, x1), normal_y(w, y1),
                    normal_x(w, x2), normal_y(w, y2), c);
      end;
    end;
  end;

procedure graphic_line(w: windows; x1, y1, x2, y2: real; c: byte);
  begin
    if DoClipping then
      graphic_clippingline(w, x1, y1, x2, y2, c)
    else
      normal_line(normal_x(w, x1), normal_y(w, y1),
                  normal_x(w, x2), normal_y(w, y2), c);
  end;

procedure graphic_cls;
  begin
    normal_cls;
  end;

procedure graphic_clearwindow(w: windows);
  begin
    with windows_array[w].userworld do begin
      normal_box(normal_x(w, left), normal_y(w, bottom),
                 normal_x(w, right), normal_y(w, top), 0,0);
    end;
  end;

begin
  for i := 1 to NUMBER_OF_GRAPHIC_WINDOWS do windows_array[i].used := False;
  DoClipping := True;
end.
(*
{$R-,S+,I+,D+,T-,F-,V+,B-,N+,L- }
program sample;
uses
  doslib4, graphic;
{$IFOPT N+}
type
  real = extended;
{$ENDIF}
var
  viewport, userworld: rectangle;
  w: array [1..4] of windows;
  N, i, j: integer;
  r, theta, delta_theta, x, y: real;

procedure QuitProgram;
  begin
    halt(1);
  end;

begin
  ClrScr;
  SetRectangle(userworld, -4.0, -4.0, 4.0, 4.0);

  SetRectangle(viewport, 0.0, 0.0, 1.0, 1.0);
  w[1] := graphic_openwindow(viewport, userworld);
  if w[1] = WINDOW_FULL then QuitProgram;

  SetRectangle(viewport, -1.0, 0.0, 0.0, 1.0);
  w[2] := graphic_openwindow(viewport, userworld);
  if w[2] = WINDOW_FULL then QuitProgram;

  SetRectangle(viewport, -1.0, -1.0, 0.0, 0.0);
  w[3] := graphic_openwindow(viewport, userworld);
  if w[3] = WINDOW_FULL then QuitProgram;

  SetRectangle(viewport, 0.0, -1.0, 1.0, 0.0);
  w[4] := graphic_openwindow(viewport, userworld);
  if w[4] = WINDOW_FULL then QuitProgram;

  writeln('Ϥޤ礦');
  N := 720;
  r := 2.0;
  theta := 0.0; delta_theta := PI * 2 / N;
  for i := 0 to N do begin
    for j := 1 to 4 do begin
      x := r * cos(theta); y := r * sin(j * theta);
      graphic_pset(w[j], x, y, j);
    end;
    theta := theta + delta_theta;
  end;
  for j := 1 to 4 do begin
    for i := 1 to 8096 do ;
    graphic_clearwindow(w[j]);
  end;
  graphic_cls;
end.
*)
