/*
 * tplot.c - plot handler for mterm
 *   written by I.Terada for a terminal mode of N88 BASIC
 *   modified a little by M.Katsurada for mterm
 */

/*
 * TERMINAL  =	MTERM98
 * #TERMINAL =	MTERM16
 * CFLAGS    =	-O
 * CC        =	cc
 * 
 * mplot   : tplot.o parse.o
 * 	$(CC) -o mplot tplot.o parse.o
 * 
 * tplot.o : tplot.c
 * 	$(CC) $(CFLAGS) -c tplot.c -D$(TERMINAL)
 * 
 * parse.o : parse.c
 * 	$(CC) $(CFLAGS) -c parse.c
 */

#ifndef MTERM98
#ifndef MTERM16
#define MTERM98
#endif
#endif

#ifdef  MTERM98
#define COM1_DEFAULT    0x1e;
#define COM2_DEFAULT    0x1f;
#endif

#ifdef  MTERM16
#define COM1_DEFAULT    0x1b;   /* was 0xfd */
#define COM2_DEFAULT    0x1f;
#endif

/* global variables */
int COM1 = COM1_DEFAULT;
int COM2 = COM2_DEFAULT;

#include <stdio.h>
#include <ctype.h>

static short int style = 0xffff;  /*  16-bit signed integer  */

static struct item {
    char *str;
    short int val;
} modtbl[] = {
    "solid",        0,
    "dotted",       1,
    "dotdashed",    2,
    "shortdashed",  3,
    "longdashed",   4,
    (char *) 0,     0
};

#define DEFAULT_XWIDTH	4096
#define DEFAULT_YHEIGHT	4096

static int xleft = 0,
	   xright = DEFAULT_XWIDTH,
	   yup = DEFAULT_YHEIGHT,
	   ydown = 0,
	   xwidth = DEFAULT_XWIDTH,
	   yheight = DEFAULT_YHEIGHT;

#define XCONV(x) (((x) - xleft) * 399 / xwidth + 120)
#define YCONV(y) (((y) - ydown) * 399 / yheight)

/* --------------------- plot(3) compatible --------------------- */

int openpl();
int circle();
int erase();
int linemod();
int line();
int move();
int cont();
int point();
int space();
int label();

/* --------------------- static functions ------------------------ */

static int set_com();
static int putint();
static int htoi();
static int stoi();

/* --------------------- plot(3) compatible --------------------- */

int
openpl()
{
    set_com();
/*
    fprintf(stderr, "%c%cGi\n", COM1, COM2);
    fprintf(stderr, "%c%cSCREEN 3 0 0 1\n", COM1, COM2);
    fprintf(stderr, "%c%cVIEW 120,0,519,399\n", COM1, COM2);
    xleft = ydown = 0;
    xright = yup = 4096;
*/
}

int
move(x, y)
int x, y;
{
    printf("%c%cGm", COM1, COM2);
    putint(XCONV(x)); putint(YCONV(y));
}

int
cont(x, y)
int x, y;
{
    printf("%c%cGn", COM1, COM2);
    putint(XCONV(x)); putint(YCONV(y));
}

int
point(x, y)
int x, y;
{
    printf("%c%cGp", COM1, COM2);
    putint(XCONV(x)); putint(YCONV(y));
}

int
line(x0, y0, x1, y1)
int x0, y0, x1, y1;
{
    printf("%c%cGl", COM1, COM2);
    putint(XCONV(x0)); putint(YCONV(y0));
    putint(XCONV(x1)); putint(YCONV(y1));
}

int
label(s)
char *s;
{
    printf("%c%cGt%s\n", COM1, COM2, s);
    fflush(stdout);
}

int
erase()
{
    printf("%c%cGe", COM1, COM2);
    fflush(stdout);
}

int
linemod(s)
char *s;
{
    struct item *itemp;

    for (itemp = modtbl; itemp -> str; itemp++) {
        if (strcmp(s,itemp -> str) == 0) {
            style = itemp -> val;
            printf("%c%cGf", COM1, COM2);
            putint(style);
            return;
        }
    }
}

int
space(x0, y0, x1, y1)
int x0, y0, x1, y1;
{
    xleft = x0; ydown = y0; xright = x1; yup = y1;
    xwidth = xright - xleft; yheight = yup - ydown;
}

int
circle(x, y, r)
int x, y, r;
{
    printf("%c%cGc", COM1, COM2);
    putint(XCONV(x)); putint(YCONV(y));
    putint(r * 399 / (xright - xleft));
    putint(r * 399 / (yup - ydown));
}

int
closepl() { }

/* ------------------------------------------------------------- */

static int
putint(n)
int n;
{
    putchar(((n >> 5) & 31) | 0x60);
    putchar((n & 31) | 0x60);
    fflush(stdout);
}

static int
htoi(s)
char *s;
{
    int c, work = 0;

    while (c = *s++) {
        work *= 16;
        if (isdigit(c))
            work += (c - '0');
        else if (c >= 'A' && c <= 'F')
            work += (c - 'A' + 10);
        else if (c >= 'a' && c <= 'f')
            work += (c - 'a' + 10);
        else
            return (work >> 4);
    }
    return work;
}

static int
stoi(s)
char *s;
{
    if (s == NULL) return 0;
    while (isspace(*s)) s++;
    if ((s[0] == '0') && (s[1] == 'x' || s[1] == 'X'))
        return htoi(&s[2]);
    else
        return atoi(s);
}

static int
set_com()
{
    int com1, com2;
    char *getenv();

    if ((com1 = stoi(getenv("MTERMCOM1"))) 
     && (com2 = stoi(getenv("MTERMCOM2")))) {
        COM1 = com1; COM2 = com2;
    }
    return 0;
}
