/*
 * unplot.c -- dump plot(5) data
 *	Its origin is parse.c which is written by I.Terada.
 *	Modified by M.Katsurada on 1 June 1992.
 */

#include <stdio.h>

#define	MYBUFSIZE	1024

/* specify the byte order (default is HIGHBYTEFIRST) */
#ifdef HIGHBYTEFIRST
#define	DEFAULT_HIGHBYTEFIRST	1
#else
#define	DEFAULT_HIGHBYTEFIRST	0
#endif

/* specify whether generate C code */
#ifdef	GENERATE_C
#define	DEFAULT_GENERATE_C	1
#else
#define DEFAULT_GENERATE_C	0
#endif

static int firstbyte_is_high	= DEFAULT_HIGHBYTEFIRST;
static int generate_c		= DEFAULT_GENERATE_C;

/* prototype */
void unplot(FILE *);
short int get2b(FILE *);
void delnewline(char *);
int redirected(int);

int main(int argc, char **argv)
{
    FILE *fp;

    if (argc == 1)
	fp = stdin;
    else if ((fp = fopen(argv[1], "r")) == NULL) {
	fprintf(stderr, "cannot open %s\n", argv[1]);
	exit(1);
    }

    if (generate_c) {
	printf("/* This program is generated by %s. */\n", argv[0]);
	puts("/* Please compile this with `-lplot'. */");
	puts("#include <stdio.h>");
	puts("main()");
	puts("{");
	puts("openpl();");
    }

    unplot(fp);

    if (generate_c) {
	puts("closepl();"); puts("}");
    }

    return 0;
}

/* parser of the intermediate code */
void unplot(FILE *fp)
{
    char s[MYBUFSIZE];	/*  buffer for string  */
    short int t0, t1, t2, t3, t4, op;

    while ((op = fgetc(fp)) >= 0) {
        switch (op) {
            case 'm':
                t0 = get2b(fp);
		printf("move(%d,%d);\n", t0,get2b(fp));
                break;
            case 'n':
                t0 = get2b(fp);
		printf("cont(%d,%d);\n", t0,get2b(fp));
                break;
            case 'p':
                t0 = get2b(fp);
		printf("point(%d,%d);\n", t0,get2b(fp));
                break;
            case 'l':
                t0 = get2b(fp); t1 = get2b(fp);
                t2 = get2b(fp);
		printf("line(%d,%d,%d,%d);\n", t0,t1,t2,get2b(fp));
                break;
            case 't':
		fgets(s,MYBUFSIZE,fp); delnewline(s);
		printf("label(\"%s\");\n", s);
                break;
            case 'e':
		printf("erase();\n");
                break;
            case 'f':
		fgets(s,MYBUFSIZE,fp); delnewline(s);
		printf("linemod(\"%s\");\n", s);
                break;
            case 's':
                t0 = get2b(fp); t1 = get2b(fp);
                t2 = get2b(fp);
		printf("space(%d,%d,%d,%d);\n", t0,t1,t2,get2b(fp));
                break;
	    case 'T': /* GNU extension */
		fgetc(fp); fgetc(fp);
		fgets(s,MYBUFSIZE,fp); delnewline(s);
		printf("label(\"%s\");\n", s);
		break;
            case 'c':
                t0 = get2b(fp); t1 = get2b(fp);
		printf("circle(%d,%d,%d);\n", t0,t1,get2b(fp));
                break;
            case 'a':
                t0 = get2b(fp); t1 = get2b(fp);
                t2 = get2b(fp); t3 = get2b(fp);
                t4 = get2b(fp);
		printf("arc(%d,%d,%d,%d,%d,%d);\n", t0,t1,t2,t3,t4,get2b(fp));
                break;
	    case (-1):
		fprintf(stderr, "-1\n");
		break;
	    default:
		printf("undefined op=%c(0x%02x)\n", op, op);
		if (redirected(1))
		    fprintf(stderr, "undefined op=%c(0x%02x)\n", op, op);
        }
    }
}

short int get2b(FILE *fp)
{
    int firstbyte;
    firstbyte = getc(fp);
    if (firstbyte_is_high)
	return 256 * firstbyte + getc(fp);
    else
	return 256 * getc(fp) + firstbyte;
}

void delnewline(char *s)
{
    int len = strlen(s) - 1;
    if (len >= 0 && s[len] == '\n') s[len] = 0;
}

/* ******************************************************************** */

#include <sys/types.h>
#include <sys/stat.h>

int redirected(int fd)
{
	struct stat buf;
	fstat(fd, &buf);
	switch (buf.st_mode & S_IFMT) {
	    case S_IFIFO:
	    case S_IFREG:
		return 1;
	    case S_IFCHR:
		return 0;
	    default:
		fprintf(stderr, "redirected(): fd=%d's file type %o\n",
			fd, buf.st_mode & S_IFMT);
		return -1;
	}
}
